'''
Plum Lightpad Python Library
https://github.com/heathbar/plum-lightpad-python

Published under the MIT license - See LICENSE file for more details.
'''

import sys
import base64
import requests

class PlumCloud():
    """Interact with Plum Cloud"""

    def __init__(self, username, password):
        auth = base64.b64encode(("%s:%s" % (username, password)).encode())
        self.headers = {
            "User-Agent": "Plum/2.3.0 (iPhone; iOS 9.2.1; Scale/2.00)",
            "Authorization": "Basic %s" % (auth.decode()),
        }

    def fetch_houses(self):
        """Lookup details for devices on the plum servers"""
        try:
            url = "https://production.plum.technology/v2/getHouses"
            return requests.get(url, headers=self.headers).json()
        except IOError:
            print("Unable to login to Plum cloud servers.")
            sys.exit(5)

    def fetch_house(self, house_id):
        """Lookup details for a given house id"""
        url = "https://production.plum.technology/v2/getHouse"
        data = {"hid": house_id}
        return self.__post(url, data)

    def fetch_room(self, room_id):
        """Lookup details for a given room id"""
        url = "https://production.plum.technology/v2/getRoom"
        data = {"rid": room_id}
        return self.__post(url, data)

    def fetch_logical_load(self, llid):
        """Lookup details for a given logical load"""
        url = "https://production.plum.technology/v2/getLogicalLoad"
        data = {"llid": llid}
        return self.__post(url, data)

    def fetch_lightpad(self, lpid):
        """Lookup details for a given lightpad"""
        url = "https://production.plum.technology/v2/getLightpad"
        data = {"lpid":lpid}
        return self.__post(url, data)

    def __post(self, url, data):
        return requests.post(url, headers=self.headers, json=data).json()

def fetch_all_the_things(username, password):
    """Fetch all info from cloud"""
    cloud = PlumCloud(username, password)
    info = {}

    houses = cloud.fetch_houses()
    for house in houses:
        house_details = cloud.fetch_house(house)

        info[house] = {
            "name": house_details["house_name"],
            "timezone": house_details["local_tz"],
            "latlon": house_details["latlong"],
            "location": house_details["location"],
            "token": house_details["house_access_token"],
            "rooms": {}
        }

        for room_id in house_details["rids"]:
            room = cloud.fetch_room(room_id)
            info[house]["rooms"][room_id] = {
                "name": room["room_name"],
                "logical_loads": {}
            }

            for llid in room["llids"]:
                load = cloud.fetch_logical_load(llid)

                info[house]["rooms"][room_id]["logical_loads"][llid] = {
                    "name": load["logical_load_name"],
                    "lightpads": {}
                }

                for lpid in load["lpids"]:
                    lightpad = cloud.fetch_lightpad(lpid)
                    info[house]["rooms"][room_id]["logical_loads"][llid]["lightpads"][lpid] = {
                        "name": lightpad["lightpad_name"],
                        "provisioned": lightpad["is_provisioned"],
                        "custom_gestures": lightpad["custom_gestures"],
                        "config": lightpad["config"]
                    }
    return info
    


'''
Plum Lightpad Python Library
https://github.com/heathbar/plum-lightpad-python

Published under the MIT license - See LICENSE file for more details.
'''

import time
import hashlib
import requests

from . import plumdiscovery
from . import plumcloud

class Plum:
    """Interact with Plum Lightpad devices"""

    def __init__(self, username, password):
        print("")
        self.local_devices = plumdiscovery.discover()
        cloud_data = plumcloud.fetch_all_the_things(username, password)
        self.loads = self.__collate_logical_loads(cloud_data, self.local_devices)

    def get_logical_loads(self):
        return self.loads

    def turn_on(self, llid):
        """Turn on a logical load"""
        self.set_level(llid, 255)

    def turn_off(self, llid):
        """Turn off a logical load"""
        self.set_level(llid, 0)

    def set_level(self, llid, level):
        """Turn on a logical load"""

        if llid in self.loads:
            # loop through lightpads until one works
            for lpid in self.loads[llid]["lightpads"]:
                try:
                    lightpad = self.loads[llid]["lightpads"][lpid]
                    url = "https://%s:%s/v2/setLogicalLoadLevel" % (lightpad["ip"], lightpad["port"])
                    data = {
                        "level": level,
                        "llid": llid
                    }
                    response = self.__post(url, data, self.loads[llid]["token"])

                    if response.status_code is 200:
                        return

                except IOError:
                    print('error')

    def __post(self, url, data, token):
        headers = {
            "User-Agent": "Plum/2.3.0 (iPhone; iOS 9.2.1; Scale/2.00)",
            "X-Plum-House-Access-Token": token
        }
        return requests.post(url, headers=headers, json=data, verify=False)

    def __collate_logical_loads(self, cloud_data, devices):
        """Make a list of all logical loads from the cloud with only the lightpads found on the current network"""

        collated_loads = {}
        sha = hashlib.new("sha256")

        for house_id in cloud_data:
            rooms = cloud_data[house_id]["rooms"]

            for room_id in rooms:
                logical_loads = cloud_data[house_id]["rooms"][room_id]["logical_loads"]

                for load_id in logical_loads:
                    load = cloud_data[house_id]["rooms"][room_id]["logical_loads"][load_id]

                    sha.update(cloud_data[house_id]["token"].encode())
                    token = sha.hexdigest()

                    collated_loads[load_id] = {
                        "name": load["name"],
                        "token": token,
                        "lightpads": {}
                    }

                    for lpid in load["lightpads"]:
                        if lpid in devices:
                            collated_loads[load_id]["lightpads"][lpid] = load["lightpads"][lpid]
                            collated_loads[load_id]["lightpads"][lpid]["ip"] = devices[lpid]["ip"]
                            collated_loads[load_id]["lightpads"][lpid]["port"] = devices[lpid]["port"]

        return collated_loads



import os
import re
from setuptools import setup

REQUIRES = []

PACKAGE_NAME = 'plumlightpad'

setup(
    name = PACKAGE_NAME,
    version = "0.0.4",
    author = "Heath Paddock",
    author_email = "hp@heathpaddock.com",
    description = ("A python package that interacts with the Plum Lightpad"),
    license = "MIT",
    keywords = ["plum", "lightpad"],
    url = "https://github.com/heathbar/plum-lightpad",
    packages=['plumlightpad'],
    include_package_data=True,
    install_requires=REQUIRES,
    classifiers=[
        'Intended Audience :: Developers',
        "Development Status :: 3 - Alpha",
        "Topic :: Home Automation",
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.4',
        "License :: OSI Approved :: MIT License",
    ],
)


'''
Plum Lightpad Python Library
https://github.com/heathbar/plum-lightpad-python

Published under the MIT license - See LICENSE file for more details.
'''

from socket import socket, timeout, AF_INET, SOCK_DGRAM, SOL_SOCKET, SO_REUSEADDR, SO_BROADCAST

def discover():
    """Broadcast a query on the network to find all Plum Lightpads"""

    devices = {}
    i = 0
    while i < 2:
        discovery_socket = socket(AF_INET, SOCK_DGRAM)
        discovery_socket.bind(("", 50000))
        discovery_socket.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
        discovery_socket.setsockopt(SOL_SOCKET, SO_BROADCAST, 1)
        discovery_socket.sendto("PLUM".encode("UTF-8"), ("255.255.255.255", 43770))
        discovery_socket.settimeout(5)
        try:
            while True:
                data, source_ip = discovery_socket.recvfrom(1024)
                info = data.decode("UTF-8").split(" ")
                lpid = info[2]

                if lpid not in devices:
                    lightpad = {}
                    lightpad.update({"ip": source_ip[0]})
                    lightpad.update({"port": info[3]})
                    devices.update({lpid: lightpad})
        except timeout:
            pass
        finally:
            discovery_socket.close()
            i += 1

    return devices
